@@ -2,10 +2,15 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE QuasiQuotes #-}
 
--- | Conversion between Chart and Markup representations.
+-- | Conversion between 'ChartOptions' and 'Markup' representations.
 module Chart.Markup
   ( Markup (..),
     ChartOptions (..),
+    forgetHud,
+    asChartTree,
+    vertCO,
+    horiCO,
+    stackCO,
     markupChartOptions,
     markupChartTree,
     markupChart,
@@ -15,15 +20,16 @@ module Chart.Markup
     writeChartOptions,
     CssOptions (..),
     defaultCssOptions,
-    CssPreferColorScheme (..),
+    PreferColorScheme (..),
     cssPreferColorScheme,
     fillSwitch,
-    CssShapeRendering (..),
+    ShapeRendering (..),
     markupCssOptions,
     MarkupOptions (..),
     defaultMarkupOptions,
     encodeNum,
     encodePx,
+    defaultCssFontFamilies,
   )
 where
 
@@ -43,6 +49,7 @@ import Data.Text (Text)
 import Data.Text.Encoding (decodeUtf8, encodeUtf8)
 import GHC.Generics
 import MarkupParse
+import NumHask.Space
 import Optics.Core hiding (element)
 import Prelude
 
@@ -75,27 +82,28 @@ encodePx = strToUtf8 . show . (floor :: Double -> Int)
 
 -- | Convert a ChartTree to markup
 --
--- >>> lineExample & view #charts & markupChartTree & markdown_ Compact Xml
+-- >>> lineExample & view #chartTree & markupChartTree & markdown_ Compact Xml
 -- "<g class=\"line\"><g stroke-width=\"0.0150\" stroke=\"rgb(2%, 73%, 80%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0,-1.0 1.0,-1.0 2.0,-5.0\"/></g><g stroke-width=\"0.0150\" stroke=\"rgb(2%, 29%, 48%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0,0 2.8,-3.0\"/></g><g stroke-width=\"0.0150\" stroke=\"rgb(66%, 7%, 55%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0.5,-4.0 0.5,0\"/></g></g>"
 markupChartTree :: ChartTree -> Markup
 markupChartTree cs =
   maybe xs' (\l -> element "g" [Attr "class" (encodeUtf8 l)] xs') label
   where
-    (ChartTree (Node (label, cs') xs)) = filterChartTree (not . isEmptyChart) cs
+    (ChartTree (Node (label, cs') xs)) = filterChartTree (not . isEmptyChart . chartData) cs
     xs' = mconcat $ fmap markupChart cs' <> (markupChartTree . ChartTree <$> xs)
 
-markupText :: TextStyle -> Text -> Point Double -> Markup
-markupText s t p@(Point x y) = element "text" as (frame' <> content c)
+markupText :: Style -> Text -> Point Double -> Markup
+markupText s t p@(Point x y) = frame' <> element "text" as (bool (contentRaw c) (content c) (EscapeText == view #escapeText s))
   where
     as =
       uncurry Attr
         <$> [ ("x", encodeNum x),
               ("y", encodeNum $ -y)
             ]
-          <> maybeToList ((\x' -> ("transform", toRotateText x' p)) <$> (s ^. #rotation))
+          <> maybeToList ((\x' -> ("transform", toRotateText x' p)) <$> view #rotation s)
+    -- This is very late for a chart creation. It is here so that the chart doesn't undergo scaling and thus picks up the local size of the text, less the border size of the frame.
     frame' = case view #frame s of
       Nothing -> Markup mempty
-      Just f -> markupChart (RectChart (f & over #borderSize (* view #size s)) [styleBoxText s t p])
+      Just f -> markupChart (Chart (f & over #borderSize (* view #size s)) (RectData [styleBoxText (s & set #frame Nothing) t p]))
     c = encodeUtf8 t
 
 -- | Markup a text rotation about a point in radians.
@@ -109,7 +117,7 @@ markupText s t p@(Point x y) = element "text" as (frame' <> content c)
 -- - flip y dimension
 toRotateText :: Double -> Point Double -> ByteString
 toRotateText r (Point x y) =
-  "rotate(" <> encodeNum (-r * 180 / pi) <> ", " <> encodeNum x <> ", " <> encodeNum (-y) <> ")"
+  "rotate(" <> encodeNum (-(r * 180 / pi)) <> ", " <> encodeNum x <> ", " <> encodeNum (-y) <> ")"
 
 toScaleText :: Double -> ByteString
 toScaleText x =
@@ -130,17 +138,20 @@ markupRect (Rect x z y w) =
 
 -- | Convert a Chart to Markup
 --
--- >>> lineExample & view #charts & foldOf charts' & head & markupChart & markdown_ Compact Xml
--- "<g stroke-width=\"0.0150\" stroke=\"rgb(2%, 73%, 80%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0,-1.0 1.0,-1.0 2.0,-5.0\"/></g>"
+-- >>> import MarkupParse
+-- >>> import Optics.Core
+-- >>> import Control.Category ((>>>))
+-- >>> lineExample & toListOf (#chartTree % charts') & mconcat & fmap (markupChart >>> markdown_ Compact Xml)
+-- ["<g stroke-width=\"0.0150\" stroke=\"rgb(2%, 73%, 80%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0,-1.0 1.0,-1.0 2.0,-5.0\"/></g>","<g stroke-width=\"0.0150\" stroke=\"rgb(2%, 29%, 48%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0,0 2.8,-3.0\"/></g>","<g stroke-width=\"0.0150\" stroke=\"rgb(66%, 7%, 55%)\" stroke-opacity=\"1.0\" fill=\"none\"><polyline points=\"0.5,-4.0 0.5,0\"/></g>"]
 markupChart :: Chart -> Markup
 markupChart = uncurry (element "g") . f
   where
-    f (RectChart s xs) = (attsRect s, mconcat (markupRect <$> xs))
-    f (TextChart s xs) = (attsText s, mconcat (uncurry (markupText s) <$> xs))
-    f (GlyphChart s xs) = (attsGlyph s, mconcat (markupGlyph s <$> xs))
-    f (PathChart s xs) = (attsPath s, markupPath xs)
-    f (LineChart s xs) = (attsLine s, markupLine xs)
-    f (BlankChart _) = ([], mempty)
+    f (Chart s (RectData xs)) = (attsRect s, mconcat (markupRect <$> xs))
+    f (Chart s (TextData xs)) = (attsText s, mconcat (uncurry (markupText s) <$> xs))
+    f (Chart s (GlyphData xs)) = (attsGlyph s, mconcat (markupGlyph s <$> xs))
+    f (Chart s (PathData xs)) = (attsPath s, markupPath xs)
+    f (Chart s (LineData xs)) = (attsLine s, markupLine xs)
+    f (Chart _ (BlankData _)) = ([], mempty)
 
 markupLine :: [[Point Double]] -> Markup
 markupLine lss =
@@ -156,13 +167,13 @@ markupPath ps =
 
 -- | GlyphStyle to markup Tree
 -- Note rotation on the outside not the inside.
-markupGlyph :: GlyphStyle -> Point Double -> Markup
+markupGlyph :: Style -> Point Double -> Markup
 markupGlyph s p =
   case view #rotation s of
     Nothing -> gl
     Just r -> element "g" [Attr "transform" (toRotateText r p)] gl
   where
-    gl = markupShape_ (s ^. #shape) (s ^. #size) p
+    gl = markupShape_ (view #glyphShape s) (view #size s) p
 
 -- | Convert a dash representation from a list to text
 fromDashArray :: [Double] -> ByteString
@@ -171,73 +182,63 @@ fromDashArray xs = intercalate " " $ encodeNum <$> xs
 fromDashOffset :: Double -> ByteString
 fromDashOffset x = encodeNum x
 
-attsLine :: LineStyle -> [Attr]
+attsLine :: Style -> [Attr]
 attsLine o =
   uncurry Attr
-    <$> [ ("stroke-width", encodeNum $ o ^. #size),
-          ("stroke", showRGB $ o ^. #color),
-          ("stroke-opacity", showOpacity $ o ^. #color),
+    <$> [ ("stroke-width", encodeNum $ view #size o),
+          ("stroke", showRGB $ view #color o),
+          ("stroke-opacity", showOpacity $ view #color o),
           ("fill", "none")
         ]
       <> catMaybes
-        [(\x -> ("stroke-linecap", fromLineCap x)) <$> (o ^. #linecap)]
-      <> foldMap (\x -> [("stroke-linejoin", fromLineJoin x)]) (o ^. #linejoin)
-      <> foldMap (\x -> [("stroke-dasharray", fromDashArray x)]) (o ^. #dasharray)
-      <> foldMap (\x -> [("stroke-dashoffset", fromDashOffset x)]) (o ^. #dashoffset)
+        [(\x -> ("stroke-linecap", fromLineCap x)) <$> view #lineCap o]
+      <> foldMap (\x -> [("stroke-linejoin", fromLineJoin x)]) (view #lineJoin o)
+      <> foldMap (\x -> [("stroke-dasharray", fromDashArray x)]) (view #dasharray o)
+      <> foldMap (\x -> [("stroke-dashoffset", fromDashOffset x)]) (view #dashoffset o)
 
-attsRect :: RectStyle -> [Attr]
+attsRect :: Style -> [Attr]
 attsRect o =
   uncurry Attr
-    <$> [ ("stroke-width", encodeNum $ o ^. #borderSize),
-          ("stroke", showRGB $ o ^. #borderColor),
-          ("stroke-opacity", showOpacity $ o ^. #borderColor),
-          ("fill", showRGB $ o ^. #color),
-          ("fill-opacity", showOpacity $ o ^. #color)
+    <$> [ ("stroke-width", encodeNum $ view #borderSize o),
+          ("stroke", showRGB $ view #borderColor o),
+          ("stroke-opacity", showOpacity $ view #borderColor o),
+          ("fill", showRGB $ view #color o),
+          ("fill-opacity", showOpacity $ view #color o)
         ]
 
 -- | TextStyle to [Attr]
-attsText :: TextStyle -> [Attr]
+attsText :: Style -> [Attr]
 attsText o =
   uncurry Attr
     <$> [ ("stroke-width", "0.0"),
           ("stroke", "none"),
-          ("fill", showRGB $ o ^. #color),
-          ("fill-opacity", showOpacity $ o ^. #color),
-          ("font-size", encodeNum $ o ^. #size),
-          ("text-anchor", toTextAnchor $ o ^. #anchor)
+          ("fill", showRGB $ view #color o),
+          ("fill-opacity", showOpacity $ view #color o),
+          ("font-size", encodeNum $ view #size o),
+          ("text-anchor", fromTextAnchor $ view #textAnchor o)
         ]
-  where
-    toTextAnchor :: Anchor -> ByteString
-    toTextAnchor AnchorMiddle = "middle"
-    toTextAnchor AnchorStart = "start"
-    toTextAnchor AnchorEnd = "end"
 
 -- | GlyphStyle to [Attr]
-attsGlyph :: GlyphStyle -> [Attr]
+attsGlyph :: Style -> [Attr]
 attsGlyph o =
   uncurry Attr
-    <$> [ ("stroke-width", encodeNum sw),
-          ("stroke", showRGB $ o ^. #borderColor),
-          ("stroke-opacity", showOpacity $ o ^. #borderColor),
-          ("fill", showRGB $ o ^. #color),
-          ("fill-opacity", showOpacity $ o ^. #color)
+    <$> [ ("stroke-width", encodeNum $ view #borderSize o),
+          ("stroke", showRGB $ view #borderColor o),
+          ("stroke-opacity", showOpacity $ view #borderColor o),
+          ("fill", showRGB $ view #color o),
+          ("fill-opacity", showOpacity $ view #color o)
         ]
-      <> foldMap ((: []) . (,) "transform" . toTranslateText) (o ^. #translate)
-  where
-    sw = case o ^. #shape of
-      PathGlyph _ NoScaleBorder -> o ^. #borderSize
-      PathGlyph _ ScaleBorder -> min 0.2 (o ^. #borderSize / o ^. #size)
-      _ -> o ^. #borderSize
+      <> foldMap ((: []) . (,) "transform" . toTranslateText) (view #translate o)
 
 -- | PathStyle to [Attr]
-attsPath :: PathStyle -> [Attr]
+attsPath :: Style -> [Attr]
 attsPath o =
   uncurry Attr
-    <$> [ ("stroke-width", encodeNum $ o ^. #borderSize),
-          ("stroke", showRGB $ o ^. #borderColor),
-          ("stroke-opacity", showOpacity $ o ^. #borderColor),
-          ("fill", showRGB $ o ^. #color),
-          ("fill-opacity", showOpacity $ o ^. #color)
+    <$> [ ("stroke-width", encodeNum $ view #borderSize o),
+          ("stroke", showRGB $ view #borderColor o),
+          ("stroke-opacity", showOpacity $ view #borderColor o),
+          ("fill", showRGB $ view #color o),
+          ("fill-opacity", showOpacity $ view #color o)
         ]
 
 -- | includes a flip of the y dimension.
@@ -259,7 +260,7 @@ markupShape_ SquareGlyph s p =
   markupRect (move p ((s *) <$> one :: Rect Double))
 markupShape_ (RectSharpGlyph x') s p =
   markupRect (move p (scale (Point s (x' * s)) one :: Rect Double))
-markupShape_ (RectRoundedGlyph x' rx ry) s p = emptyElem "rect" as
+markupShape_ (RectRoundedGlyph x' rx' ry') s p = emptyElem "rect" as
   where
     as =
       uncurry Attr
@@ -267,8 +268,8 @@ markupShape_ (RectRoundedGlyph x' rx ry) s p = emptyElem "rect" as
               ("height", encodeNum $ w - y),
               ("x", encodeNum x),
               ("y", encodeNum $ -w),
-              ("rx", encodeNum rx),
-              ("ry", encodeNum ry)
+              ("rx", encodeNum rx'),
+              ("ry", encodeNum ry')
             ]
     (Rect x z y w) = move p (scale (Point s (x' * s)) one)
 markupShape_ (TriangleGlyph (Point xa ya) (Point xb yb) (Point xc yc)) s p =
@@ -293,36 +294,44 @@ markupShape_ VLineGlyph s (Point x y) =
   emptyElem "polyline" [Attr "points" $ encodeNum x <> "," <> encodeNum (-(y - s / 2)) <> "\n" <> encodeNum x <> "," <> encodeNum (-(y + s / 2))]
 markupShape_ HLineGlyph s (Point x y) =
   emptyElem "polyline" [Attr "points" $ encodeNum (x - s / 2) <> "," <> encodeNum (-y) <> "\n" <> encodeNum (x + s / 2) <> "," <> encodeNum (-y)]
-markupShape_ (PathGlyph path _) s p =
+markupShape_ (PathGlyph path) s p =
   emptyElem "path" (uncurry Attr <$> [("d", path), ("transform", toTranslateText p <> " " <> toScaleText s)])
 
 -- | Create the classic SVG element
 --
--- >>> header 100 one (element_ "foo" []) & markdown_ Compact Xml
--- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"100\" height=\"100\" viewBox=\"-0.5 -0.5 1.0 1.0\"><foo></foo></svg>"
-header :: Double -> Rect Double -> Markup -> Markup
+-- >>> header (Just 300) (Rect (-0.75) 0.75 (-0.5) 0.5) (element_ "foo" []) & markdown_ Compact Xml
+-- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"450\" height=\"300\" viewBox=\"-0.75 -0.5 1.5 1.0\"><foo></foo></svg>"
+header :: Maybe Double -> Rect Double -> Markup -> Markup
 header markupheight viewbox content' =
   element
     "svg"
     ( uncurry Attr
-        <$> [ ("xmlns", "http://www.w3.org/2000/svg"),
-              ("xmlns:xlink", "http://www.w3.org/1999/xlink"),
-              ("width", encodePx w''),
-              ("height", encodePx h'),
-              ("viewBox", encodeNum x <> " " <> encodeNum (-w) <> " " <> encodeNum (z - x) <> " " <> encodeNum (w - y))
-            ]
+        <$> ( [ ("xmlns", "http://www.w3.org/2000/svg"),
+                ("xmlns:xlink", "http://www.w3.org/1999/xlink")
+              ]
+                <> widthAndHeight
+                <> [ ("viewBox", encodeNum x <> " " <> encodeNum (-w) <> " " <> encodeNum (z - x) <> " " <> encodeNum (w - y))
+                   ]
+            )
     )
     content'
   where
     (Rect x z y w) = viewbox
     Point w' h = width viewbox
-    Point w'' h' = Point (markupheight / h * w') markupheight
+    widthAndHeight = case markupheight of
+      Nothing -> []
+      Just h' ->
+        [ ("width", encodePx w''),
+          ("height", encodePx h')
+        ]
+        where
+          w'' = h' / h * w'
 
 -- | CSS prefer-color-scheme text snippet
 --
 -- >>> cssPreferColorScheme (light, dark) PreferHud
 -- "svg {\n  color-scheme: light dark;\n}\n{\n  .canvas g, .title g, .axisbar g, .ticktext g, .tickglyph g, .ticklines g, .legendContent g text {\n    fill: rgb(5%, 5%, 5%);\n  }\n  .ticklines g, .tickglyph g, .legendBorder g {\n    stroke: rgb(5%, 5%, 5%);\n  }\n  .legendBorder g {\n    fill: rgb(94%, 94%, 94%);\n  }\n}\n@media (prefers-color-scheme:dark) {\n  .canvas g, .title g, .axisbar g, .ticktext g, .tickglyph g, .ticklines g, .legendContent g text {\n    fill: rgb(94%, 94%, 94%);\n  }\n  .ticklines g, .tickglyph g, .legendBorder g {\n    stroke: rgb(94%, 94%, 94%);\n  }\n  .legendBorder g {\n    fill: rgb(5%, 5%, 5%);\n  }\n}"
-cssPreferColorScheme :: (Colour, Colour) -> CssPreferColorScheme -> ByteString
+cssPreferColorScheme :: (Colour, Colour) -> PreferColorScheme -> ByteString
 cssPreferColorScheme (cl, cd) PreferHud =
   [i|svg {
   color-scheme: light dark;
@@ -392,9 +401,10 @@ fillSwitch (colorNormal, colorPrefer) prefer item =
 -- | Markup options.
 --
 -- >>> defaultMarkupOptions
--- MarkupOptions {markupHeight = 300.0, cssOptions = CssOptions {shapeRendering = NoShapeRendering, preferColorScheme = PreferHud, cssExtra = ""}, renderStyle = Compact}
+-- MarkupOptions {markupHeight = Just 300.0, chartAspect = FixedAspect 1.5, cssOptions = CssOptions {shapeRendering = NoShapeRendering, preferColorScheme = PreferHud, fontFamilies = "\nsvg { font-family: system-ui,-apple-system,\"Segoe UI\",Roboto,\"Helvetica Neue\",Arial,\"Noto Sans\",\"Liberation Sans\",sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",\"Segoe UI Symbol\",\"Noto Color Emoji\";\n}\n\nticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,\"Liberation Mono\",\"Courier New\",monospace;\n}\n\n", cssExtra = ""}, renderStyle = Compact}
 data MarkupOptions = MarkupOptions
-  { markupHeight :: Double,
+  { markupHeight :: Maybe Double,
+    chartAspect :: ChartAspect,
     cssOptions :: CssOptions,
     renderStyle :: RenderStyle
   }
@@ -402,13 +412,25 @@ data MarkupOptions = MarkupOptions
 
 -- | The official markup options
 defaultMarkupOptions :: MarkupOptions
-defaultMarkupOptions = MarkupOptions 300 defaultCssOptions Compact
+defaultMarkupOptions = MarkupOptions (Just 300) (FixedAspect 1.5) defaultCssOptions Compact
 
--- | CSS shape rendering options
-data CssShapeRendering = UseGeometricPrecision | UseCssCrisp | NoShapeRendering deriving (Show, Eq, Generic)
+-- | default fonts.
+defaultCssFontFamilies :: ByteString
+defaultCssFontFamilies =
+  [i|
+svg { font-family: system-ui,-apple-system,"Segoe UI",Roboto,"Helvetica Neue",Arial,"Noto Sans","Liberation Sans",sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol","Noto Color Emoji";
+}
+
+ticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,"Liberation Mono","Courier New",monospace;
+}
+
+|]
+
+-- | CSS glyphShape rendering options
+data ShapeRendering = UseGeometricPrecision | UseCssCrisp | NoShapeRendering deriving (Show, Eq, Generic)
 
 -- | CSS prefer-color-scheme options
-data CssPreferColorScheme
+data PreferColorScheme
   = -- | includes css that switches approriate hud elements between light and dark.
     PreferHud
   | PreferDark
@@ -419,12 +441,12 @@ data CssPreferColorScheme
 -- | css options
 --
 -- >>> defaultCssOptions
--- CssOptions {shapeRendering = NoShapeRendering, preferColorScheme = PreferHud, cssExtra = ""}
-data CssOptions = CssOptions {shapeRendering :: CssShapeRendering, preferColorScheme :: CssPreferColorScheme, cssExtra :: ByteString} deriving (Show, Eq, Generic)
+-- CssOptions {shapeRendering = NoShapeRendering, preferColorScheme = PreferHud, fontFamilies = "\nsvg { font-family: system-ui,-apple-system,\"Segoe UI\",Roboto,\"Helvetica Neue\",Arial,\"Noto Sans\",\"Liberation Sans\",sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",\"Segoe UI Symbol\",\"Noto Color Emoji\";\n}\n\nticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,\"Liberation Mono\",\"Courier New\",monospace;\n}\n\n", cssExtra = ""}
+data CssOptions = CssOptions {shapeRendering :: ShapeRendering, preferColorScheme :: PreferColorScheme, fontFamilies :: ByteString, cssExtra :: ByteString} deriving (Show, Eq, Generic)
 
 -- | No special shape rendering and default hud responds to user color scheme preferences.
 defaultCssOptions :: CssOptions
-defaultCssOptions = CssOptions NoShapeRendering PreferHud mempty
+defaultCssOptions = CssOptions NoShapeRendering PreferHud defaultCssFontFamilies mempty
 
 -- | Convert CssOptions to Markup
 markupCssOptions :: CssOptions -> Markup
@@ -432,49 +454,82 @@ markupCssOptions css =
   elementc "style" [] $
     cssPreferColorScheme (light, dark) (view #preferColorScheme css)
       <> markupShapeRendering (view #shapeRendering css)
+      <> view #fontFamilies css
       <> view #cssExtra css
 
 -- | CSS shape rendering text snippet
-markupShapeRendering :: CssShapeRendering -> ByteString
+markupShapeRendering :: ShapeRendering -> ByteString
 markupShapeRendering UseGeometricPrecision = "svg { shape-rendering: geometricPrecision; }"
 markupShapeRendering UseCssCrisp = "svg { shape-rendering: crispEdges; }"
 markupShapeRendering NoShapeRendering = mempty
 
--- | A product type representing charts, hud options and markup options, which can be transformed into 'Markup'.
+-- | A product type consisting of a 'ChartTree', 'HudOptions' and 'MarkupOptions', which is what you need to create 'Markup'.
 data ChartOptions = ChartOptions
   { markupOptions :: MarkupOptions,
     hudOptions :: HudOptions,
-    charts :: ChartTree
+    chartTree :: ChartTree
   }
   deriving (Generic, Eq, Show)
 
+-- | Processes the hud options and turns them into charts, rescales the existing charts, resets the hud options to mempty, and turns on 'ScalePArea' in chart styles.
+--
+-- Note that this is a destructive operation, and, in particular, that
+--
+-- > view #chartTree (forgetHud (mempty & set #chartTree c)) /= c
+forgetHud :: ChartOptions -> ChartOptions
+forgetHud co =
+  co
+    & set #hudOptions mempty
+    & set #chartTree (addHud (view (#markupOptions % #chartAspect) co) (view #hudOptions co) (view #chartTree co))
+    & set (#chartTree % charts' % each % #chartStyle % #scaleP) ScalePArea
+
+-- | Convert ChartOptions to a ChartTree, forgetting the original hud and chart data
+asChartTree :: ChartOptions -> ChartTree
+asChartTree = view #chartTree . forgetHud
+
+-- | Horizontally stack a list of ChartOptions (proceeding to the right), at the supplied Align and with the supplied gap intercalated.
+horiCO :: Align -> Double -> [ChartOptions] -> ChartOptions
+horiCO align gap cs = mempty @ChartOptions & set #chartTree (hori align gap (asChartTree <$> cs))
+
+-- | Vertically stack a list of ChartOptions (proceeding upwards), at the supplied Align and with the supplied gap intercalated.
+vertCO :: Align -> Double -> [ChartOptions] -> ChartOptions
+vertCO align gap cs = mempty @ChartOptions & set #chartTree (vert align gap (asChartTree <$> cs))
+
+-- | Stack a list of ChartOptions horizontally, then vertically (proceeding downwards which is opposite to the usual coordinate reference system but intuitively the way people read charts)
+stackCO :: Int -> Align -> Align -> Double -> [ChartOptions] -> ChartOptions
+stackCO n alignV alignH gap cs = mempty @ChartOptions & set #chartTree (stack n alignV alignH gap (asChartTree <$> cs))
+
 -- | Convert ChartOptions to Markup
 --
 -- >>> markupChartOptions (ChartOptions (defaultMarkupOptions & #cssOptions % #preferColorScheme .~ PreferNormal) mempty mempty) & markdown_ Compact Xml
--- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"450\" height=\"300\" viewBox=\"-0.75 -0.5 1.5 1.0\"><style></style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
+-- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"300\" height=\"300\" viewBox=\"-0.5 -0.5 1.0 1.0\"><style>\nsvg { font-family: system-ui,-apple-system,\"Segoe UI\",Roboto,\"Helvetica Neue\",Arial,\"Noto Sans\",\"Liberation Sans\",sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",\"Segoe UI Symbol\",\"Noto Color Emoji\";\n}\n\nticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,\"Liberation Mono\",\"Courier New\",monospace;\n}\n\n</style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
 markupChartOptions :: ChartOptions -> Markup
 markupChartOptions co =
   header
     (view (#markupOptions % #markupHeight) co)
     viewbox
     ( markupCssOptions (view (#markupOptions % #cssOptions) co)
-        <> markupChartTree csAndHud
+        <> markupChartTree ctFinal
     )
   where
-    viewbox = singletonGuard (view styleBox' csAndHud)
-    csAndHud = addHud (view #hudOptions co) (view #charts co)
+    viewbox = view safeStyleBox' ctFinal
+    ctFinal =
+      projectChartTreeWith
+        (view (#markupOptions % #chartAspect) co)
+        (view #hudOptions co)
+        (view #chartTree co)
 
 -- | Render ChartOptions to an SVG ByteString
 --
 -- >>> encodeChartOptions (ChartOptions (defaultMarkupOptions & #cssOptions % #preferColorScheme .~ PreferNormal) mempty mempty)
--- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"450\" height=\"300\" viewBox=\"-0.75 -0.5 1.5 1.0\"><style></style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
+-- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"300\" height=\"300\" viewBox=\"-0.5 -0.5 1.0 1.0\"><style>\nsvg { font-family: system-ui,-apple-system,\"Segoe UI\",Roboto,\"Helvetica Neue\",Arial,\"Noto Sans\",\"Liberation Sans\",sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",\"Segoe UI Symbol\",\"Noto Color Emoji\";\n}\n\nticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,\"Liberation Mono\",\"Courier New\",monospace;\n}\n\n</style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
 encodeChartOptions :: ChartOptions -> ByteString
 encodeChartOptions co = markdown_ (view (#markupOptions % #renderStyle) co) Xml $ markupChartOptions co
 
 -- | Render ChartOptions to an SVG Text snippet
 --
 -- >>> renderChartOptions (ChartOptions (defaultMarkupOptions & #cssOptions % #preferColorScheme .~ PreferNormal) mempty mempty)
--- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"450\" height=\"300\" viewBox=\"-0.75 -0.5 1.5 1.0\"><style></style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
+-- "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"300\" height=\"300\" viewBox=\"-0.5 -0.5 1.0 1.0\"><style>\nsvg { font-family: system-ui,-apple-system,\"Segoe UI\",Roboto,\"Helvetica Neue\",Arial,\"Noto Sans\",\"Liberation Sans\",sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",\"Segoe UI Symbol\",\"Noto Color Emoji\";\n}\n\nticktext { font-family: SFMono-Regular,Menlo,Monaco,Consolas,\"Liberation Mono\",\"Courier New\",monospace;\n}\n\n</style><g class=\"chart\"></g><g class=\"hud\"></g></svg>"
 renderChartOptions :: ChartOptions -> Text
 renderChartOptions = decodeUtf8 . encodeChartOptions
 
