@@ -9,6 +9,7 @@ module Data.Path
     pointPath,
     movePath,
     scalePath,
+    projectPath,
     projectPaths,
     pathBoxes,
     pathBox,
@@ -44,11 +45,10 @@ module Data.Path
 where
 
 import Chart.Data
-import Control.Foldl qualified as L
-import Control.Monad.State.Lazy
 import GHC.Generics
 import Geom2D.CubicBezier qualified as B
 import NumHask.Prelude
+import NumHask.Space hiding (rw, rx, ry, rz)
 
 -- $setup
 --
@@ -106,16 +106,7 @@ scalePath x (ArcP i p) = ArcP i (fmap (x *) p)
 
 -- | Project a list of connected PathDatas from one Rect (XY plave) to a new one.
 projectPaths :: Rect Double -> Rect Double -> [PathData Double] -> [PathData Double]
-projectPaths new old ps =
-  flip evalState zero $
-    mapM
-      ( \p -> do
-          x <- get
-          let d = projectPath new old x p
-          put (pointPath d)
-          pure d
-      )
-      ps
+projectPaths new old ps = snd $ mapAccumL (\s a -> let d = projectPath new old s a in (pointPath d, d)) zero ps
 
 -- | Project a PathData from one Rect (XY plave) to a new one.
 projectPath ::
@@ -193,7 +184,7 @@ data ArcCentroid a = ArcCentroid
 --
 -- >>> let p = ArcPosition (Point 0 0) (Point 1 0) (ArcInfo (Point 1 0.5) (pi/4) False True)
 -- >>> arcCentroid p
--- ArcCentroid {centroid = Point 0.20952624903444356 -0.48412291827592724, radius = Point 1.0 0.5, cphi = 0.7853981633974483, ang0 = 1.3753858999692936, angdiff = -1.823476581936975}
+-- ArcCentroid {centroid = Point 0.20952624903444356 (-0.48412291827592724), radius = Point 1.0 0.5, cphi = 0.7853981633974483, ang0 = 1.3753858999692936, angdiff = -1.823476581936975}
 arcCentroid :: (Ord a, FromInteger a, TrigField a, ExpField a) => ArcPosition a -> ArcCentroid a
 arcCentroid (ArcPosition p1@(Point x1 y1) p2@(Point x2 y2) (ArcInfo rad phi' large' clockwise')) = ArcCentroid c (Point rx ry) phi' ang1 angd
   where
@@ -211,12 +202,12 @@ arcCentroid (ArcPosition p1@(Point x1 y1) p2@(Point x2 y2) (ArcInfo rad phi' lar
     cx = (x1 + x2) / 2 + cos phi' * cx' - sin phi' * cy'
     cy = (y1 + y2) / 2 + sin phi' * cx' + cos phi' * cy'
     c = Point cx cy
-    ang1 = angle (Point (-(cx' - x1') / rx) (-(cy' - y1') / ry))
-    ang2 = angle (Point (-(cx' + x1') / rx) (-(cy' + y1') / ry))
+    ang1 = angle (Point (-((cx' - x1') / rx)) (-((cy' - y1') / ry)))
+    ang2 = angle (Point (-((cx' + x1') / rx)) (-((cy' + y1') / ry)))
     angd' = ang2 - ang1
     angd =
       bool 0 (2 * pi) (not clockwise' && angd' < 0)
-        + bool 0 (-2 * pi) (clockwise' && angd' > 0)
+        + bool 0 (-(2 * pi)) (clockwise' && angd' > 0)
         + angd'
 
 -- | Convert from an ArcCentroid to an ArcPosition specification.
@@ -242,7 +233,7 @@ arcPosition (ArcCentroid c r phi' ang1 angd) =
 -- | Ellipse formulae
 --
 -- >>> ellipse zero (Point 1 2) (pi/6) pi
--- Point -0.8660254037844388 -0.4999999999999997
+-- Point (-0.8660254037844388) (-0.4999999999999997)
 --
 -- Compare this "elegent" definition from [stackexchange](https://math.stackexchange.com/questions/426150/what-is-the-general-equation-of-the-ellipse-that-is-not-in-the-origin-and-rotate)
 --
@@ -258,10 +249,8 @@ ellipse c r phi' theta = c + (rotate phi' |. (r * ray theta))
 
 -- | compute the bounding box for an arcBox
 --
--- >>> let p = ArcPosition (Point 0 0) (Point 1 0) (ArcInfo (Point 1 0.5) (pi/4) False True)
--- >>> import Data.FormatN
--- >>> fmap (fixed (Just 3)) (arcBox p)
--- Rect "-0.000" "1.000" "-0.000" "0.306"
+-- > let p = ArcPosition (Point 0 0) (Point 1 0) (ArcInfo (Point 1 0.5) (pi/4) False True)
+-- > arcBox p
 arcBox :: ArcPosition Double -> Rect Double
 arcBox p = unsafeSpace1 pts
   where
@@ -291,7 +280,7 @@ arcBox p = unsafeSpace1 pts
 arcDerivs :: Point Double -> Double -> (Double, Double)
 arcDerivs (Point rx ry) phi' = (thetax1, thetay1)
   where
-    thetax1 = atan2 (-sin phi' * ry) (cos phi' * rx)
+    thetax1 = atan2 (-(sin phi' * ry)) (cos phi' * rx)
     thetay1 = atan2 (cos phi' * ry) (sin phi' * rx)
 
 -- | Quadratic bezier curve expressed in positional terms.
@@ -332,7 +321,7 @@ quadPolar (QuadPosition start' end control) = QuadPolar start' end control'
 -- > quadPolar . quadPosition == id
 --
 -- >>> quadPosition $ quadPolar (QuadPosition (Point 0 0) (Point 1 1) (Point 2 (-1)))
--- QuadPosition {qposStart = Point 0.0 0.0, qposEnd = Point 1.0 1.0, qposControl = Point 2.0 -0.9999999999999998}
+-- QuadPosition {qposStart = Point 0.0 0.0, qposEnd = Point 1.0 1.0, qposControl = Point 2.0 (-0.9999999999999998)}
 quadPosition :: (TrigField a) => QuadPolar a -> QuadPosition a
 quadPosition (QuadPolar start' end control) = QuadPosition start' end control'
   where
@@ -341,7 +330,7 @@ quadPosition (QuadPolar start' end control) = QuadPosition start' end control'
 -- | The quadratic bezier equation
 --
 -- >>> quadBezier (QuadPosition (Point 0 0) (Point 1 1) (Point 2 (-1))) 0.33333333
--- Point 0.9999999933333332 -0.33333333333333326
+-- Point 0.9999999933333332 (-0.33333333333333326)
 quadBezier :: (FromInteger a, ExpField a) => QuadPosition a -> a -> Point a
 quadBezier (QuadPosition start' end control) theta =
   (1 - theta)
@@ -369,7 +358,7 @@ quadDerivs (QuadPosition start' end control) = [x', y']
 -- | Bounding box for a QuadPosition
 --
 -- >>> quadBox (QuadPosition (Point 0 0) (Point 1 1) (Point 2 (-1)))
--- Rect 0.0 1.3333333333333335 -0.33333333333333337 1.0
+-- Rect 0.0 1.3333333333333335 (-0.33333333333333337) 1.0
 quadBox :: QuadPosition Double -> Rect Double
 quadBox p = unsafeSpace1 pts
   where
@@ -424,7 +413,7 @@ cubicPolar (CubicPosition start' end control1 control2) = CubicPolar start' end
 -- > cubicPolar . cubicPosition == id
 --
 -- >>> cubicPosition $ cubicPolar (CubicPosition (Point 0 0) (Point 1 1) (Point 1 (-1)) (Point 0 2))
--- CubicPosition {cposStart = Point 0.0 0.0, cposEnd = Point 1.0 1.0, cposControl1 = Point 1.0 -1.0, cposControl2 = Point 1.6653345369377348e-16 2.0}
+-- CubicPosition {cposStart = Point 0.0 0.0, cposEnd = Point 1.0 1.0, cposControl1 = Point 1.0 (-1.0), cposControl2 = Point 1.6653345369377348e-16 2.0}
 cubicPosition :: (Eq a, TrigField a, ExpField a) => CubicPolar a -> CubicPosition a
 cubicPosition (CubicPolar start' end control1 control2) = CubicPosition start' end control1' control2'
   where
@@ -478,7 +467,7 @@ cubicDerivs
 -- | Bounding box for a CubicPosition
 --
 -- >>> cubicBox (CubicPosition (Point 0 0) (Point 1 1) (Point 1 (-1)) (Point 0 2))
--- Rect 0.0 1.0 -0.20710678118654752 1.2071067811865475
+-- Rect 0.0 1.0 (-0.20710678118654752) 1.2071067811865475
 cubicBox :: CubicPosition Double -> Rect Double
 cubicBox p = unsafeSpace1 pts
   where
@@ -494,7 +483,8 @@ pathBoxes :: [PathData Double] -> Maybe (Rect Double)
 pathBoxes [] = Nothing
 pathBoxes (x : xs) =
   Just $
-    L.fold (L.Fold step begin snd) xs
+    snd $
+      foldl' step begin xs
   where
     begin :: (Point Double, Rect Double)
     begin = (pointPath x, singleton (pointPath x))
